#Diagramme solaire en azimut et hauteur

import matplotlib.pyplot as plt
import numpy as np
from math import sin,  cos,  tan, asin, atan, acos

# -----------------------------------

plt.rcParams.update({"text.usetex": True}) 
# plt.rcParams["font.family"] = "Roboto" # ou autre police installée

plt.rcParams["font.size"] = 14

# Paramètres à personnaliser
phi=np.radians(49.5)#latitude
couleur_lignesH='royalblue'
# -----------------------------------

hauteurmax=90+23.4-np.degrees(phi)#hauteur méridienne au 21 juin, pour l'échelle de hauteur
maxH=np.degrees(acos(-tan(phi)*tan(np.radians(23.4))))#angle horaire maxi au 21 juin, pour les heures de lever/coucher
maxH=int(maxH/15)*15

maxAz=np.degrees(acos(-sin(np.radians(23.4)/cos(phi))))#azimut maximal au lever/coucher, pour l'axe x
maxAz=int(maxAz/20+1)*20


 # ------------------------FONCTIONS
 
def calcul_hauteur(D,H):
    # renvoie la hauteur du Soleil, d'après la déclinaison D et l'angle horaire H
    return np.degrees(asin(sin(np.radians(D))*sin(phi)+cos(np.radians(D))*cos(phi)*cos(np.radians(H))))

def calcul_azimut(D, H) :
    # renvoie l'azimut corrigé
    Az=np.degrees(atan(sin(np.radians(H))/(sin(phi)*cos(np.radians(H))-cos(phi)*tan(np.radians(D)))))
    if Az<0 and H>0:
        Az=Az+180
    elif Az>0 and H<0:
        Az=Az-180
    return Az

"""
vectorization des fonctions précédentes, pour qu'elles puissent 
être appliquées à une liste de valeurs, l'autre variable étant fixe.
"""
liste_hauteur=np.vectorize(calcul_hauteur)
liste_azimut=np.vectorize(calcul_azimut)
# -------------------------------------------

#paramètres du graphique :
fig=plt.figure(figsize=(12, 9), tight_layout=True)
ax=plt.subplot()
plt.xticks(np.arange(-150, 200, 50))#graduations chiffrées en azimut
plt.xlim(-maxAz, maxAz)
plt.title(r"$\textsf{ \textbf{Diagramme solaire}}$")
plt.text(0.005, 0.993, r'Latitude $\mathbf{%.1f}$°'%(np.degrees(phi)), color='red', va='top', fontsize=16, transform=ax.transAxes, bbox=dict(facecolor='white', edgecolor='black'))
plt.ylim(0, int(hauteurmax+5))
plt.xlabel("Azimut (°)")
plt.ylabel("Hauteur (°)")

#pts cardinaux :
cardinaux={'N-E':-135,'Est': -90,'S-E': -45,
           'S-O': 45,'Ouest': 90,'N-O': 135}

for direction in cardinaux:
    plt.text(cardinaux[direction], -2.5, direction,
             va='top', ha='center', rotation=90)


#Grille :
minor_xticks = np.arange(-maxAz, maxAz, 10)#espaces de la grille
minor_yticks = np.arange(0, int(hauteurmax+5), 1)#espaces de la grille
ax.set_xticks(minor_xticks, minor=True)
ax.set_yticks(minor_yticks, minor=True)
ax.grid(which='minor', alpha=0.5)
plt.grid()

#Tracé des lignes horaires :-------------------------------------------------
decl=np.arange(-23.4, 23.4, 0.05)
for H in np.arange(-maxH, maxH+7.5, 7.5):
    if H.is_integer()==False:#demi-heures
        props=dict(c='gray', alpha=0.5, lw=1)#ligne fine et grise
    else :#heures pleines
        props=dict(alpha=1, c=couleur_lignesH)#ligne épaisse, en couleur (non grise)
    X=liste_azimut(decl,H)
    Y=liste_hauteur(decl,H)
    plt.plot(X,Y, **props)
    if H.is_integer():
        if H>=0 :#chiffres des heures
            prop_chiffres=dict(ha='left')
            X=1.01*max(X)
        elif H<0 :
            prop_chiffres=dict(ha='right')
            X=1.01*min(X)
        plt.text(X, 1.01*max(Y), r'$\textbf{%i}$'%(12+H/15), fontweight='bold',**prop_chiffres)


#courbes de déclinaison ::-------------------------------------------------
couleursdecl=['crimson', 'blueviolet', 'teal','orangered', 'indigo', 'darkorange', 'darkolivegreen']
datesdecl=['21 déc.', '20 jan. et 22 nov.','19 fév. et 23 oct.', '20 mar. et 23 sept.', '20 avr. et 23 aou.', '21 mai et 23 juil.', '21 juin', '']
H = np.arange(-maxH-15, maxH+15, 0.1)# liste des angles horaires
for i,  D in enumerate([-23.4, -20.15, -11.5, 0, 11.5, 20.15, 23.4]):
    X=liste_azimut(D,H)
    Y=liste_hauteur(D,H)
    plt.plot(X, Y, c=couleursdecl[i], label=datesdecl[i])

#------------------------------------------------Courbe de date supplémentaire
"""
datesup=['10 nov.']
declsup=[-17.25]#déclinaisons des dates
for i,  D in enumerate(declsup):
    X=liste_azimut(declsup,H)
    Y=liste_hauteur(declsup,H)
    plt.plot(X, Y, ls='--', label=datesup[i])
"""
#------------------------------------------------
plt.legend()

fig.savefig("DiagrammeSolaire495.png", dpi=300)
fig.savefig("DiagrammeSolaire495.pdf")
# plt.show()